/****************************************************************************
 *
 * rw/numbrw.cc - Internal classes for numeric formatting and parsing.
 *
 * $Id: numbrw.cc,v 1.22 1996/10/02 01:48:00 smithey Exp $
 *
 ***************************************************************************
 *
 * (c) Copyright 1994-1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ***************************************************************************/

#ifndef __STD_RW_NUMBRW_CC__
#define __STD_RW_NUMBRW_CC__

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __save
#      pragma __environment __header_defaults
#   endif
#endif

#ifndef _RWSTD_NO_NAMESPACE
namespace __rwstd {
#endif

// -----------------------------------------------
// Template digit_handler_base_1 member templates.
// -----------------------------------------------

template <class charT>
_RWSTD_IMP_DECL(digit_handler_base_1)<charT>::
_RWSTD_IMP_DECL(digit_handler_base_1) (const locale &loc):
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
    ctyp(use_facet<ctype<charT> >(loc)),
    dmap(_RWSTD_IMP_DECL(digit_map)<charT>::get_digit_map(ctyp)),
    punct(use_facet<numpunct<charT> >(loc))
#else
    ctyp(use_facet(loc,(ctype<charT>*)0)),
    dmap(_RWSTD_IMP_DECL(digit_map)<charT>::get_digit_map(ctyp)),
    punct(use_facet(loc,(numpunct<charT>*)0))
#endif
{ }

template <class charT>
_RWSTD_IMP_DECL(digit_handler_base_1)<charT>::
_RWSTD_IMP_DECL(digit_handler_base_1)
    (const locale &loc, const _RWSTD_IMP_DECL(punct_data)<charT> &pun):
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
    ctyp(use_facet<ctype<charT> >(loc)),
    dmap(_RWSTD_IMP_DECL(digit_map)<charT>::get_digit_map(ctyp)),
#else
    ctyp(use_facet(loc,(ctype<charT>*)0)),
    dmap(_RWSTD_IMP_DECL(digit_map)<charT>::get_digit_map(ctyp)),
#endif
    punct(pun)
{ }

// ----------------------------------------------
// Template digit_reader_base_1 member templates.
// ----------------------------------------------

template <class charT>
_RWSTD_IMP_DECL(digit_reader_base_1)<charT>::
_RWSTD_IMP_DECL(digit_reader_base_1) (const locale &loc):
    _RWSTD_IMP_DECL(digit_handler_base_1)<charT>(loc)
{ }

template <class charT>
_RWSTD_IMP_DECL(digit_reader_base_1)<charT>::
_RWSTD_IMP_DECL(digit_reader_base_1) (const locale &loc,
    const _RWSTD_IMP_DECL(punct_data)<charT> &mp):
    _RWSTD_IMP_DECL(digit_handler_base_1)<charT>(loc,mp)
{ }

// ---------------------------------------
// Template digit_reader member templates.
// ---------------------------------------

template <class charT,class InputIterator>
_RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>::
_RWSTD_IMP_DECL(digit_reader)(InputIterator& i,InputIterator& e,ios_base &b):
    _RWSTD_IMP_DECL(digit_reader_base_1)<charT>(b.getloc()),
    io(b), in(i), end(e)
{ }

template <class charT,class InputIterator>
_RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>::
_RWSTD_IMP_DECL(digit_reader)(InputIterator& i,InputIterator& e,ios_base &b,
    const _RWSTD_IMP_DECL(punct_data)<charT> &mp):
    _RWSTD_IMP_DECL(digit_reader_base_1)<charT>(b.getloc(),mp),
    io(b), in(i), end(e)
{ }

template <class charT,class InputIterator>
char* _RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>::
    get_digit_string (char *dpos)
{
  char *eod=dpos;
  int i;

  while (!at_end() && (i=this->dmap.eval(*in))>=0 && i<this->radix) {
    if (eod==this->digits+sizeof this->digits)
      this->error|=this->too_many_digits;
    else
      *eod++=(char) i;
    ++in;
  }

  return eod;
}

template <class charT,class InputIterator>
char* _RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>::
    get_int_digits (void)
{
  charT c;
  char *eod=this->digits;
  const charT* ctype_punct=this->dmap.get_punct();

  if (!at_end() && this->is_signed)
    // If a sign is present, set negative if it's a minus sign.
    do {
      if ((c=*in)==ctype_punct[this->minus])
        this->negative=true;
      else if (c!=ctype_punct[this->plus])
        break;
      // Eat white space after the sign.  Standard seems to mandate this
      // regardless of whether ios_base::skipws is set.
      do
        ++in;
      while (!at_end() && this->ctyp.is(ctype_base::space,*in));
      this->advanced=true;
    } while (0);

  if (!this->radix) {
    switch (io.flags() & ios_base::basefield) {
     case ios_base::dec:
      this->radix=10;
      break;
     case ios_base::hex:
      this->radix=16;
      break;
     case ios_base::oct:
      this->radix=8;
      break;
     default:
      // No radix was forced by the ios_base flags, so poke around for a radix
      // specifier on the front of the input value.
      this->radix=10;
      if (!at_end() && *in==ctype_punct[this->zero]) {
        this->radix=8;
        ++in;
        *eod++=0;
        this->advanced=true;
        if (!at_end() &&
            ((c=*in)==ctype_punct[this->x] || c==ctype_punct[this->X]))
        {
          this->radix=16;
          ++in;
          eod--; // Leading 0 was not a digit after all.
        }
      }
    }
  }

  grouping=this->get_grouping();
  if (!*grouping)
    eod=get_digit_string(eod);
  else {
    gpos=groups;
    thousands_sep=get_thousands_sep();
    eod=get_digit_groups(eod);
    if (!this->error && --gpos>groups) {
      // Digit grouping is optional, but if present it must be right.
      const char *gdef=grouping;
      do {
        if (*gdef!=*gpos && *gdef!=CHAR_MAX)
          break;
        else
          if (gdef[1])
            gdef++;
      } while (--gpos>groups);

      if (gpos>groups || *gpos>*gdef)
        this->error|=this->bad_grouping;
    }
  }

  if (eod>this->digits)
    this->advanced=true;
  return eod;
}

template <class charT,class InputIterator>
char* _RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>::
    get_float_digits (void)
{
  charT c;
  this->radix=10;
  char *eod=get_int_digits();

  const charT *ctyp_punct=this->dmap.get_punct();

  this->frac_beg=eod;
  if (!this->error && !at_end() && *in==get_decimal_point()) {
    in++;
    eod=get_digit_string(eod);
  }

  if (eod==this->digits)
    this->error|=this->no_digits;

  this->exp_beg=eod;
  this->exp_negative=false;
  if (!this->error && !at_end() &&
      ((c=*in)==ctyp_punct[this->e] || c==ctyp_punct[this->E]))
  {
    in++;
    if (at_end())
      this->error|=this->bad_exponent;
    else {
      if ((c=*in)==ctyp_punct[this->plus])
        in++;
      else if (c==ctyp_punct[this->minus]) {
        this->exp_negative=true;
        in++;
      }

      int save_radix=this->radix;
      this->radix=10;
      eod=get_digit_string(eod);
      this->radix=save_radix;
      if (eod==this->exp_beg)
        this->error|=this->bad_exponent;
    }
  }

  return eod;
}

template <class charT,class InputIterator>
char *_RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>::
    get_pointer_digits (void)
{
  this->radix=16;
  char *eod=get_int_digits();
  return eod;
}

template <class charT,class InputIterator>
char* _RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>::
    get_digit_groups (char *dpos)
{
  char *eod=get_digit_string(dpos);

  if (gpos==groups+sizeof groups)
    this->error|=this->too_many_groups;
  else {
    int i=eod-dpos;
    if (i >= CHAR_MAX)
      this->error|=this->group_too_long;
    else {
      *gpos++=i;
      if (i!=0 && !at_end() && *in==thousands_sep) {
        ++in;
        eod=get_digit_groups(eod);
      }
    }
  }

  return eod;
}

template <class charT,class InputIterator>
int _RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>::
    get_keyword (const _RWSTD_IMP_DECL(keyword_map)<charT> &map)
{
  const _RWSTD_IMP_DECL(keyword_def)<charT> *canp=map.defs_;
  const _RWSTD_IMP_DECL(keyword_def)<charT> *endp=map.defs_+map.num_defs_;
  _RWSTD_IMP_DECL(keyword_def)<charT> cans[40],*ansp=cans;

  charT c;
  const charT *p;
  int result;

  while (1) {
    if (at_end()) {
      for ( ; canp<endp; canp++)
        if (!*canp->s)
          return canp->v;
      return -1;
    }
    c=*in;
    result=-1;
    for ( ; canp<endp; canp++) {
      p=canp->s;
      if (c==*p) {
        ansp->s=++p;
        ansp->v=canp->v;
        ansp++;
      } else if (*p==0)
        result=ansp->v; // abbreviation
    }

    if (ansp==cans)
      return result;
    in++;
    if (ansp==cans+1) {
      for (p=cans[0].s; *p; p++) {
        if (at_end() || *in!=*p)
          return -1;
        in++;
      }
      return cans[0].v;
    }

    endp=ansp;
    ansp=cans;
    canp=cans;
  }
}

// --------------------------------------
// Template time_reader member templates.
// --------------------------------------

template <class charT,class InputIterator>
_RWSTD_IMP_DECL(time_reader)<charT,InputIterator>::
_RWSTD_IMP_DECL(time_reader)
    (InputIterator& in,InputIterator& end,ios_base& io):
    _RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>(in,end,io),
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
    timp(use_facet<_RWSTD_IMP_DECL(timepunct)<charT> >(io.getloc()))
#else
//  timp(use_facet(io.getloc(),(_RWSTD_IMP_DECL(timepunct)<charT>*)0))
    timp(ms_idiocy(io))
#endif
{
  // Don't recognize signs on any numeric input.
  this->is_signed=false;
}

template <class charT,class InputIterator>
bool _RWSTD_IMP_DECL(time_reader)<charT,InputIterator>::
    get_time_pattern (const string_type &patt,tm *time)
{
  enum {
    edit_year = 0x01,
    edit_month = 0x02,
    edit_century = 0x04
  };

  int post_edits=0;
  const charT *p=patt.c_str(),*patt_end=p+patt.length();
  for ( ; p<patt_end; p++) {
    switch (this->ctyp.narrow(*p,' ')) {
     case '%':
      {
        int *iresult;
        const _RWSTD_IMP_DECL(keyword_map)<charT> *keywords=NULL;

        switch (this->ctyp.narrow(*++p,' ')) {
         case 'a':
         case 'A':
          // Day of week (abbreviated or not).
          keywords=&get_day_map(timp);
          iresult=&time->tm_wday;
          break;
         case 'b':
         case 'B':
          // Month name (abbreviated or not).
          keywords=&get_month_map(timp);
          iresult=&time->tm_mon;
          break;
         case 'm':
          // Numeric month, comes in 1 higher than the number we want.
          post_edits|=edit_month;
          iresult=&time->tm_mon;
          break;
         case 'c':
          // Full date, time and year in default format.
          return get_time_pattern(get_patt_string(timp,2),time);
         case 'd':
          // Day of month.
          iresult=&time->tm_mday;
          break;
         case 'H':
          // Hours.
          iresult=&time->tm_hour;
          break;
         case 'M':
          // Minutes.
          iresult=&time->tm_min;
          break;
         case 'S':
          // Seconds.
          iresult=&time->tm_sec;
          break;
         case 'Y':
          // Year with Century.
          post_edits|=edit_year;
          iresult=&time->tm_year;
          break;
         case 'y':
          // 2-digit Year without Century.
          post_edits|=(edit_year|edit_century);
          iresult=&time->tm_year;
          break;
         default:
          // All other format characters are not supported on input.
          return false;
        }

        if (keywords) {
          if ((*iresult=get_keyword(*keywords))<0)
            return false;
        } else
          *iresult=to_ulong(this->get_int_digits());
      }
      break;

     default:
      if (!this->at_end() && *this->in==*p)
        this->in++;
      else
        return false;
    }

    if (this->error)
      return false;
  }

  if (post_edits&edit_year)
    if (post_edits&edit_century && time->tm_year<100)
      ; // 2-digit year is already relative to 1900
    else
      time->tm_year-=1900;

  if (post_edits&edit_month)
    time->tm_mon--;

  return true;
}

// -----------------------------------------------------------
// Template class money_reader_base_1<charT> member templates.
// -----------------------------------------------------------

template <class charT>
_RWSTD_IMP_DECL(money_reader_base_1)<charT>::
_RWSTD_IMP_DECL(money_reader_base_1)
    (_RWSTD_IMP_DECL(digit_reader_base_1)<charT> &r,
     const _RWSTD_IMP_DECL(moneypunct_data)<charT> &mp):
       _RWSTD_IMP_DECL(money_handler_base_1)<charT>(mp), reader(r)
{ }

template <class charT>
void _RWSTD_IMP_DECL(money_reader_base_1)<charT>::get_money_string
    (string_type &result,const char *eod)
{
  int len=eod-reader.digits;
  if (reader.negative)
    len++;

  result.resize(0);
  result.reserve(len);

  if (reader.negative)
    result+=reader.dmap.punct_chars[_RWSTD_IMP_DECL(digit_map_base)::minus];

  const char *d=reader.digits;
  const char *end=eod-1;
  while (d<end && *d==0) d++;
  for ( ; d<=end; d++)
    result+=reader.dmap.digit_chars[*d];
}

// -------------------------------------------------------------------------
// Template class money_reader_base_2<charT,InputIterator> member templates.
// -------------------------------------------------------------------------

template <class charT,class InputIterator>
_RWSTD_IMP_DECL(money_reader_base_2)<charT,InputIterator>::
_RWSTD_IMP_DECL(money_reader_base_2) (InputIterator &i,InputIterator &e,
    ios_base &b,const _RWSTD_IMP_DECL(moneypunct_data)<charT> &mp):
    _RWSTD_IMP_DECL(digit_reader)<charT,InputIterator>(i,e,b,mp),
    _RWSTD_IMP_DECL(money_reader_base_1)<charT>(*this,mp)
{
  this->radix=10;
  this->is_signed=false;
}

template <class charT,class InputIterator>
char* _RWSTD_IMP_DECL(money_reader_base_2)<charT,InputIterator>::
    get_money_digits (void)
{
  charT c;
  // Always parse input according to negative format.
  const money_base::pattern &patt=get_neg_format();

  // If this ends up non-NULL, it points to trailing sign char(s) that are
  // required at the end of the pattern.
  const charT *sign=NULL;

  bool need_curr=(this->io.flags()&ios_base::showbase)? true : false;
  bool got_curr=false;
  int fracs=get_frac_digits();

  char *eod=this->digits;
  const char *p=patt.field,*pend=p+sizeof patt.field-1;

  for ( ; !this->error && p<=pend; p++) {
    switch (*p) {
     case money_base::space:
     case money_base::none:
      while (!this->at_end() && this->ctyp.is(ctype_base::space,*this->in))
        this->in++;
      break;

     case money_base::symbol:
      if (!this->at_end()) {
        const charT *curr=get_curr_symbol().c_str();
        if ((c=*this->in)==*curr) {
          // We ate a character, so rest of symbol must be present.
          need_curr=true;
          do {
            this->in++;
            if (*++curr==charT('\0')) {
              got_curr=true;
              break;
            }
          } while (!this->at_end() && (c=*this->in)==*curr);
        }
      }
      if (need_curr && !got_curr)
        this->error|=this->bad_curr_symbol;
      break;

     case money_base::sign:
      if (!this->at_end()) {
        sign=get_negative_sign().c_str();
        if ((c=*this->in)==*sign) {
          this->negative=true;
          sign++;
          this->in++;
        } else {
          sign=get_positive_sign().c_str();
          if (c==*sign) {
            sign++;
            this->in++;
          } else
            sign=NULL;
        }
      }
      break;

     case money_base::value:
      eod=get_int_digits();
      if (!this->error && fracs && !this->at_end() &&
          *this->in==get_decimal_point())
      {
        this->in++;
        char *frac_start=eod;
        eod=get_digit_string(eod);
        if (!this->error) {
          fracs-=(eod-frac_start);
          if (fracs<0) {
            // Too many fractional digits in input.  We truncate.  The
            // alternative would be to return an error.
            eod+=fracs;
            fracs=0;
          }
        }
      }
      if (eod==this->digits)
        this->error|=this->no_digits;
      else if (!this->error && fracs>0) {
        // Add trailing zeros until the correct number of fractional digits
        // is reached.
        if (this->digits+sizeof this->digits-eod < fracs)
          this->error|=this->too_many_digits;
        else {
          do *eod++=0;
          while (--fracs);
        }
      }
      break;
    }
  }

  if (sign && !this->error)
    // We ate one charT of a sign earlier, rest of sign must be present at end.
    while (*sign)
      if (!this->at_end() && *this->in==*sign++)
        this->in++;
      else {
        this->error|=this->bad_sign;
        break;
      }

  this->frac_beg=this->exp_beg=eod;
  return eod;
}

// -----------------------------------------------------------------
// Template money_reader<charT,Intl,InputIterator> member templates.
// -----------------------------------------------------------------

template <class charT,bool Intl,class InputIterator>
_RWSTD_IMP_DECL(money_reader)<charT,Intl,InputIterator>::
_RWSTD_IMP_DECL(money_reader) (InputIterator &i,InputIterator &e,ios_base &b):
    _RWSTD_IMP_DECL(money_reader_base_2)<charT,InputIterator>
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
      (i,e,b,use_facet<moneypunct<charT,Intl> >(b.getloc()))
#else
      (i,e,b,use_facet(b.getloc(),(moneypunct<charT,Intl>*)0))
#endif
{ }

// ----------------------------------------------
// Template digit_writer_base_1 member templates.
// ----------------------------------------------

template <class charT>
_RWSTD_IMP_DECL(digit_writer_base_1)<charT>::
_RWSTD_IMP_DECL(digit_writer_base_1)
    (ios_base &b,const locale &loc):
      _RWSTD_IMP_DECL(digit_writer_base)(b),
      _RWSTD_IMP_DECL(digit_handler_base_1)<charT>(loc)
{ }

template <class charT>
_RWSTD_IMP_DECL(digit_writer_base_1)<charT>::
_RWSTD_IMP_DECL(digit_writer_base_1)
    (ios_base &b,const _RWSTD_IMP_DECL(punct_data)<charT> &mp):
      _RWSTD_IMP_DECL(digit_writer_base)(b),
      _RWSTD_IMP_DECL(digit_handler_base_1)<charT>(b.getloc(),mp)
{
  flags&=~(ios_base::floatfield|ios_base::showpos);
  flags|=ios_base::fixed;
  radix=10;
  precision=0;
}

// ---------------------------------------
// Template digit_writer member templates.
// ---------------------------------------

template <class charT,class OutputIterator>
_RWSTD_IMP_DECL(digit_writer)<charT,OutputIterator>::
_RWSTD_IMP_DECL(digit_writer) (OutputIterator &o, ios_base &b):
      _RWSTD_IMP_DECL(digit_writer_base_1)<charT>(b,b.getloc()), out(o) { }

template <class charT,class OutputIterator>
_RWSTD_IMP_DECL(digit_writer)<charT,OutputIterator>::
_RWSTD_IMP_DECL(digit_writer) (OutputIterator &o, ios_base &b,
    const _RWSTD_IMP_DECL(punct_data)<charT> &mp):
      _RWSTD_IMP_DECL(digit_writer_base_1)<charT>(b,mp), out(o) { }

template <class charT, class OutputIterator>
void _RWSTD_IMP_DECL(digit_writer)<charT,OutputIterator>::
    put_digits (charT fill)
{
  char *p=this->start;
  bool has_sign=false,has_point=false;
  if (p<this->end && (*p==' ' || *p=='-' || *p=='+'))
    has_sign=true;

  // Locate the end of the integral digits.
  char *dec;
  if (!this->fractional)
    dec=this->end;
  else {
    dec=this->start;
    if (has_sign) dec++;
    for ( ; dec<this->end; dec++)
      if (*dec<'0' || *dec>'9') {
        if (*dec!='e' && *dec!='E')
          has_point=true;
        break;
      }
  }

  // Calculate the number and pattern of separators needed if any.
  charT separator;
  int unGrouped=dec-this->start;
  if (has_sign)
    unGrouped--;
  if (this->radix==10 && this->separable) {
    unGrouped=calc_groups(unGrouped,this->get_grouping());
    if (this->num_groups)
      separator=get_thousands_sep();
  }

  // Compute the number of fill charT-s needed, and where they should be put.
  int leftFill=0,internalFill=0,rightFill=0;
  if (this->width>0) {
    int w=this->width - (this->end - this->start) - this->num_groups;
    this->width=0;
    if (w>0) {
      switch (this->adjust) {
       case _RWSTD_IMP_DECL(digit_writer_base)::left:
        rightFill=w;
        break;
       case _RWSTD_IMP_DECL(digit_writer_base)::internal:
        internalFill=w;
        break;
       default:
        leftFill=w;
      }
    }
  }

  // Widen the sign + digits + exponent string.
//charT wide_digits[sizeof _RWSTD_IMP_DECL(digit_writer_base)::buffer];
  charT wide_digits[400];
  this->ctyp.widen(this->start,this->end,wide_digits);

  // Write the widened string with fill and decorations to output.
  charT *digit_pos=wide_digits;
  while (leftFill--)
    *out++=fill;
  if (has_sign)
    *out++=*digit_pos++;    // the widened sign
  while (internalFill--)
    *out++=fill;
  while (unGrouped--)
    *out++=*digit_pos++;
  while (this->num_groups--) {
    *out++=separator;
    while (this->group[0]--)
      *out++=*digit_pos++;
    this->group++;
  }

  if (has_point) {
    *out++=get_decimal_point();
    digit_pos++;
  }

  unGrouped=this->end-dec;
  if (has_point)
    unGrouped--;
  while (unGrouped-->0)
    *out++=*digit_pos++;

  while (rightFill--)
    *out++=fill;
}

template <class charT, class OutputIterator>
void _RWSTD_IMP_DECL(digit_writer)<charT,OutputIterator>::
    put_keyword (const string_type &k, charT fill)
{
  int leftFill=0,rightFill=0;
  int n=this->width-k.length();
  if (n>0) {
    switch (this->adjust) {
     case _RWSTD_IMP_DECL(digit_writer_base)::left:
      rightFill=n;
      break;
     case _RWSTD_IMP_DECL(digit_writer_base)::internal:
     default:
      leftFill=n;
    }
    if (leftFill)
      do *out++=fill;
      while (--leftFill);
  }

  const charT *p=k.c_str(),*end=p+k.length();
  while (p<end)
    *out++=*p++;

  if (rightFill)
    do *out++=fill;
    while (--rightFill);
}

// -----------------------------------------------------
// Template money_writer_base_1<charT> member templates.
// -----------------------------------------------------

template <class charT>
_RWSTD_IMP_DECL(money_writer_base_1)<charT>::
_RWSTD_IMP_DECL(money_writer_base_1)
    (_RWSTD_IMP_DECL(digit_writer_base_1)<charT> &w,
     const _RWSTD_IMP_DECL(moneypunct_data)<charT> &mp):
    _RWSTD_IMP_DECL(money_handler_base_1)<charT>(mp), writer(w)
{ }

template <class charT>
void _RWSTD_IMP_DECL(money_writer_base_1)<charT>::put_money (charT fill)
{
  bool negative;
  if (*writer.start=='-') {
    negative=true;
    writer.start++;
  } else {
    negative=false;
    if (*writer.start=='+' || *writer.start==' ')
      writer.start++;
  }

  charT wide_digits[sizeof writer.buffer];
  writer.ctyp.widen(writer.start,writer.end,wide_digits);
  put_money_sub(wide_digits,wide_digits+(writer.end-writer.start),
      negative,fill);
}

template <class charT>
void _RWSTD_IMP_DECL(money_writer_base_1)<charT>::put_money
    (const string_type& digits,charT fill)
{
  const charT *punct =
      _RWSTD_IMP_DECL(digit_map)<charT>::get_digit_map(writer.ctyp)
      .get_punct();

  const charT *start=digits.c_str();
  bool negative;
  if (*start==punct[_RWSTD_IMP_DECL(digit_map_base)::minus]) {
    negative=true;
    start++;
  } else
    negative=false;

  const charT *end=writer.ctyp.scan_not(ctype_base::digit,start,
      digits.c_str()+digits.length());

  put_money_sub(start,end,negative,fill);
}

// --------------------------------------------------------------------
// Template money_writer_base_2<charT,OutputIterator> member templates.
// --------------------------------------------------------------------

template <class charT,class OutputIterator>
_RWSTD_IMP_DECL(money_writer_base_2)<charT,OutputIterator>::
_RWSTD_IMP_DECL(money_writer_base_2)
    (OutputIterator &os,ios_base &io,
     const _RWSTD_IMP_DECL(moneypunct_data)<charT> &mp):
    _RWSTD_IMP_DECL(digit_writer)<charT,OutputIterator> (os,io,mp),
    _RWSTD_IMP_DECL(money_writer_base_1)<charT> (*this,mp)
{ }

template <class charT, class OutputIterator>
void _RWSTD_IMP_DECL(money_writer_base_2)<charT,OutputIterator>::put_money_sub
    (const charT *start,const charT *end,bool negative,charT fill)
{
  const money_base::pattern *patt;
  const string_type *sign;
  if (negative) {
    patt=&get_neg_format();
    sign=&get_negative_sign();
  } else {
    patt=&get_pos_format();
    sign=&get_positive_sign();
  }

  int frac_digits=get_frac_digits();
  int int_digits=end-start-frac_digits;
  int unGrouped,zero_pad;

  if (int_digits<0) {
    zero_pad=-int_digits;
    int_digits=0;
  } else
    zero_pad=0;

  charT sep;
  if (int_digits>0) {
    unGrouped=calc_groups(int_digits,this->get_grouping());
    if (this->num_groups)
      sep=get_thousands_sep();
  } else
    unGrouped=0;

  const char *p,*pend=patt->field+sizeof patt->field;
  int leftFill=0,internalFill=0,rightFill=0;

  if (this->width) {
    int n=0;
    for (p=patt->field; p<pend; p++) {
      switch (*p) {
       case money_base::space:
        n++;
        break;
       case money_base::symbol:
        if (this->flags&ios_base::showbase)
          n+=get_curr_symbol().length();
        break;
       case money_base::sign:
        n+=sign->length();
        break;
       case money_base::value:
        n+=int_digits+this->num_groups;
        if (frac_digits)
          n+=frac_digits+1;
        break;
      }
    }

    if ((n-=this->width)>0) {
      switch (this->adjust) {
       case _RWSTD_IMP_DECL(digit_writer_base)::left:
        rightFill=n;
        break;
       case _RWSTD_IMP_DECL(digit_writer_base)::internal:
        internalFill=n;
        break;
       default:
        leftFill=n;
      }
    }

    this->width=0;
  }

  if (leftFill)
    do *this->out++=fill;
    while (--leftFill);

  const charT *schar=sign->c_str();
  for (p=patt->field; p<pend; p++)
    switch (*p) {
     case money_base::symbol:
      if (this->flags&ios_base::showbase)
        put_keyword(get_curr_symbol(),fill);
      break;
     case money_base::sign:
      if (*schar)
        *this->out++=*schar++;
      break;
     case money_base::value:
      while (unGrouped--)
        *this->out++=*start++;
      while (this->num_groups--) {
        *this->out++=sep;
        while ((*this->group)--)
          *this->out++=*start++;
        this->group++;
      }
      if (frac_digits) {
        *this->out++=get_decimal_point();
        while (zero_pad--) {
          frac_digits--;
          *this->out++=this->ctyp.widen('0');
        }
        while (frac_digits-->0)
          *this->out++=*start++;
      }
      break;
     case money_base::space:
      if (!internalFill) {
        *this->out++=this->ctyp.widen(' ');
        break;
      }
      // Fall through ...
     case money_base::none:
      if (internalFill)
        do *this->out++=fill;
        while (--internalFill);
      break;
    }

  while (*schar)
    *this->out++=*schar++;

  if (rightFill+=internalFill) {
    do *this->out++=fill;
    while (--rightFill);
  }
}

// ------------------------------------------------------------------
// Template money_writer<charT,Intl,OutputIterator> member templates.
// ------------------------------------------------------------------

template <class charT,bool Intl,class OutputIterator>
_RWSTD_IMP_DECL(money_writer)<charT,Intl,OutputIterator>::
_RWSTD_IMP_DECL(money_writer) (OutputIterator &os,ios_base &io):
    _RWSTD_IMP_DECL(money_writer_base_2)<charT,OutputIterator>
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
      (os,io,use_facet<moneypunct<charT,Intl> >(io.getloc()))
#else
      (os,io,use_facet(io.getloc(),(moneypunct<charT,Intl>*)0))
#endif
{ }

#ifndef _RWSTD_NO_NAMESPACE
} // namespace __rwstd
#endif

#if defined(__DECCXX)
#   ifdef __PRAGMA_ENVIRONMENT
#      pragma __environment __restore
#   endif
#endif

#endif // __STD_RW_NUMBRW_CC__
