/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright Compaq Computer Corporation, 2000                *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*	
 *	@(#)$RCSfile: dumprestore.h,v $ $Revision: 4.2.14.3 $ (DEC) $Date: 1996/02/08 18:40:32 $
 */ 
/*
 */
/*
 * (c) Copyright 1990, OPEN SOFTWARE FOUNDATION, INC.
 * ALL RIGHTS RESERVED
 */
/*
 * OSF/1 Release 1.0
 */
/*
 * Copyright (c) 1980 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *

 */
#ifndef _PROTOCOLS_DUMPRESTORE_H_
#define _PROTOCOLS_DUMPRESTORE_H_

#include <sys/types.h>
#include <ufs/dinode.h>

/*
 * TP_BSIZE is the size of file blocks on the dump tapes.
 * Note that TP_BSIZE must be a multiple of DEV_BSIZE.
 *
 * NTREC is the number of TP_BSIZE blocks that are written
 * in each tape record. HIGHDENSITYTREC is the number of
 * TP_BSIZE blocks that are written in each tape record on
 * 6250 BPI or higher density tapes.
 *
 * TP_NINDIR is the number of indirect pointers in a TS_INODE
 * or TS_ADDR record. Note that it must be a power of two.
 */
#define TP_BSIZE	1024
#define NTREC   	10
#define HIGHDENSITYTREC	32
#define TP_NINDIR	(TP_BSIZE/2)
#define LBLSIZE		16
#define NAMELEN		64
#define TP_LIST		(TP_BSIZE/2)

#define OFS_MAGIC   	(int)60011
#define NFS_MAGIC   	(int)60012
#define CHECKSUM	(int)84446

union u_spcl {
	char dummy[TP_BSIZE];
	struct	s_spcl {
		int	c_type;		    /* record type (see below) */
		time_t	c_date;		    /* date of previous dump */
		time_t	c_ddate;	    /* date of this dump */
		int	c_volume;	    /* dump volume number */
		daddr_t	c_tapea;	    /* logical block of this record */
		ino_t	c_inumber;	    /* number of inode */
		int	c_magic;	    /* magic number (see above) */
		int	c_checksum;	    /* record checksum */
		struct	dinode	c_dinode;   /* ownership and mode of inode */
		int	c_count;	    /* number of valid c_addr entries */
		char	c_addr[TP_NINDIR];  /* 1 => data; 0 => hole in inode */
		char	c_label[LBLSIZE];   /* dump label */
		int	c_level;	    /* level of this dump */
		char	c_filesys[NAMELEN]; /* name of dumpped file system */
		char	c_dev[NAMELEN];	    /* name of dumpped device */
		char	c_host[NAMELEN];    /* name of dumpped host */
		int	c_flags;	    /* additional information */
	} s_spcl;
	struct s_proplist_spcl {
		int	c_type;		    /* record type (see below) */
		time_t  c_date;             /* date of previous dump */
                time_t  c_ddate;            /* date of this dump */
                int     c_length;           /* length of property list */
                daddr_t c_tapea;            /* logical block of this record */
		ino_t	c_inumber;	    /* number of inode */
		int	c_magic;	    /* magic number (see above) */
		int	c_checksum;	    /* record checksum */
     		struct  dinode  c_dinode;   /* ownership and mode of inode */
		int	c_count;	    /* number of valid c_addr entries */
		char	c_proplist[TP_LIST];/* the property list */
	} s_proplist_spcl;

} u_spcl;
#define spcl u_spcl.s_spcl
#define spcl_proplist u_spcl.s_proplist_spcl
/*
 * special record types
 */
#define TS_TAPE         1       /* dump tape header */
#define TS_INODE        2       /* beginning of file record */
#define TS_ADDR         4       /* continuation of file record */
#define TS_BITS         3       /* map of inodes on tape */
#define TS_CLRI         6       /* map of inodes deleted since last dump */
#define TS_END          5       /* end of volume marker */
#define TS_LIST         7      /* file property list */
#if SEC_FSCHANGE
#define TS_ACL          8       /* map of acl tag to ir conversion */
#define TS_PRIV         9       /* map of privs */
#define TS_MACIL        10       /* map of mac/il tag to ir conversion */
#endif

/*
 * flag values
 */
#define DR_NEWHEADER	1	/* new format tape header */

#define	DUMPOUTFMT	"%-s %c %s"		/* for printf */
						/* name, incno, ctime(date) */
#define	DUMPINFMT	"%s %c %[^\n]\n"	/* inverse for scanf */

#endif
